#!/usr/bin/perl
#!c:\Perl\bin\perl.exe

# PL0013.pm	
# MAILER_VAR_HTML - sendet Plain- und HTML-Mails mit und ohne Anhang via SMTP oder sendmail
# EMAIL_TEST - Prft Emailadresse auf Plausibilitt
# GET_DATEI_STRING - Liest Datei als String ein (Schlrfmodus)

package EIGEN::Mail::PL0013;

use strict;
use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);

require Exporter;

@ISA       = qw(Exporter AuoLoader);
@EXPORT    = qw();
@EXPORT_OK = qw(MAILER_VAR_HTML EMAIL_TEST GET_DATEI_STRING);
$VERSION   = '1.0.0';

use Socket;

##################################################################
# Voreinstellungen:
##################################################################


##################################################################
# Subroutinen:
##################################################################

##################################################################
# MAILER_VAR_HTML
# Variables Mailprogramm, bercksichtigt SMTP und sendmail.
# Sendet HTML-Mails mit eingelagerten Bildern und Dateianhngen oder
# text/plain Emails.
# Die in einem HTML-body verwendeten Bilder drfen keine externen Bilder
# der Form http://www.domain.de/... enthalten.
# Bilder und Datei Body.html oder Body.txt mssen im gleichen Verzeichnis
# liegen.
# Bildreferenzen relativ anlegen, also z.B. <img src="Bild1.gif">.
#
# input: 
# from    - Absender-Email   z.B. 'from@efg.de'
# reply   - Antwort-Email    z.B. 'rep@abc.de''
# to      - Empfnger-Email  (Array-Referenz)
# cc      - Kopie an...      (Array-Referenz)
# z.B.
# cc      - $cc[0]   = 'cc01@domain01.de';
# cc      - ...
# cc      - $cc[N]   = 'cc01@domainN+1.de';
# cc      - $hash{cc}= \@cc;
# bcc     - Kopie an...;unsichtba) (Array-Referenz)
# subject - Betreffzeile             z.B. 'Betreff der Mail hier'
# body    - Nachrichtentext (HTML, auch mit Bildern oder plain Text)
# z.B.
# body    - 1. 'Hallo\nhier kommt meine Nachricht\n\nGru HM'
# body    - 2. '<html><head><style media="screen" type="text/css"><!--...--></style></head><body>Hier kommt der Text in HTML</body></html>'
# body    - 3. 'c:\Inetpub\wwwroot\Baurat\Verzeichnis\Datei.html bzw. .txt'
# attach  - Emailanhnge (Array-Referenz)
# z.B.
# attach  - $attach[0] = 'c:\Pfa1\Verzeichnis1\Datei1.Endung1';
# attach  - $attach[1] = 'c:\Pfa2\Verzeichnis2\Datei2.Endung2';
# attach  - ...
# attach  - $attach[N] = 'c:\PfaN+1\VerzeichnisN+1\DateiN+1.EndungN+1';
# attach  - $hash{attach} = \@attach;
# imp     - 'low', 'normal' oder 'high'
#
# %hash = (from=>$from,reply=>$rep,to=>\@to,cc=>\@cc,bcc=>\@bcc,subject=>$sub,body=>$message,attach=>$att,imp=>$imp);
#
# Aufruf:
# &MAILER_VAR_HTML(\%hash);
#
# output: "OK - ..." oder "Fehler ..."
#
##################################################################
sub MAILER_VAR_HTML
{
 my $hash = $_[0];
 my (@Image,$system,@DatNamTeile,$msg,@ImgNamTeile,$img_pfad);
 
 # Mailserver: z.b. mail.pegasus.net
 # Bitte hier eintragen:
 my $smtp_server = "Ihr.mailserver.net";
 
 use MIME::Lite;
 
 if ($ENV{'OS'} eq "Windows_NT")
 {
  $system = "smtp";
 }
 else
 {
  $system = "sendmail";
 }

 
##############################################
### Email-Adressen auf Existenz und Plausibilitt prfen

 my ($tmp);
 
### From
 if ($hash->{'from'})
 {
  unless (&EMAIL_TEST($hash->{'from'}))  {return("Fehler in Emailadresse \"From\":$hash->{'from'}");}
 }
 else
 {
  return("Fehler, keine Absender-Emailadresse angegeben!");
 }
 if ($hash->{'reply'})
 {
  unless (&EMAIL_TEST($hash->{'reply'})) {return("Fehler in Emailadresse \"Reply\":$hash->{'reply'}");}
 }
 else
 {
  $hash->{'reply'} = $hash->{'from'} ;
 }
 
### To
 if (@{$hash->{'to'}})
 {
  foreach $tmp (@{$hash->{'to'}})
  {
   unless (&EMAIL_TEST($tmp)) {return("Fehler in Emailadresse \"To\":$tmp");}
  }
 }
 else
 {
  return("Fehler, keine Empfngeradresse angegeben!");
 }

### Cc
 if ($hash->{'cc'})
 {
  foreach $tmp (@{$hash->{'cc'}})
  {
   unless (&EMAIL_TEST($tmp)) {return("Fehler in Emailadresse \"Cc\":$tmp");}
  }
 }
 
### Bcc
 if ($hash->{'bcc'})
 {
  foreach $tmp (@{$hash->{'bcc'}})
  {
   unless (&EMAIL_TEST($tmp)) {return("Fehler in Emailadresse \"Bcc\":$tmp");}
  }
 } 
 
##############################################
### Subject auf Existenz prfen

unless ($hash->{subject} || $hash->{subject} ne "") {return("Fehler, kein Betreff vorhanden.");}

##############################################
### Email-Body

 my ($body,$bodytyp);
 
### Falls Pfad + Dateiname, Body aus Datei einlesen
 if ($hash->{body} =~ m/^[cC]:\\|\.[Hh][Tt][Mm][Ll]?$|\.[Tt][Xx][Tt]$/)
 {
  if (-e $hash->{body}) {$body  = &GET_DATEI_STRING("$hash->{body}");} else {return("Fehler in Dateiangabe fr Body: $hash->{body}, Fehler: $!");}
  if ($body eq "") {return("Fehler in eingelesenem Body: Die Datei ist leer!");}
  if ($body =~ m/^\s*?<[Hh][Tt][Mm][Ll]>/ && $body =~ m/<\/[Hh][Tt][Mm][Ll]>\s*$/)
  {
   $bodytyp = 'text/html';
  }
  else
  {
   $bodytyp = 'text/plain';
  }
 }

### Body wurde direkt als HTML-Text bergeben
 elsif ($hash->{body} =~ m/^\s*?<[Hh][Tt][Mm][Ll]>/ && $hash->{body} =~ m/<\/[Hh][Tt][Mm][Ll]>\s*$/)
 {
  $body    = $hash->{body};
  $bodytyp = 'text/html';
 }

### Body wurde direkt als Text/Plain bergeben 
 else
 {
  if ($hash->{body} eq "") {return("Fehler in bergebenem Body: Der Body ist leer!");}
  $body    = $hash->{body};
  $bodytyp = 'text/plain';
 }
 
### Alle eingelagerten Bilder merken - externe Bilder verursachen Fehler!
 if ($bodytyp eq 'text/html')
 {
  @Image = $body =~ m/<img src=\"(.*?)\".*?>/g; # 1_x_1.gif Webhead250x67.gif usw.

  if (@Image) # Nur wenn Bild vorhanden
  {
   if (@Image > 0) # Nur wenn Bild vorhanden
   {
   
### Pfad zum Bild ermitteln
    if ($system eq "smtp")
    {
     @ImgNamTeile = split(/\\/,$hash->{'body'}); # Dateipfad bei "\" aufsplitten
    }
    else
    {
     @ImgNamTeile = split(/\//,$hash->{'body'}); # Dateipfad bei "/" aufsplitten
    }
    $img_pfad = substr($hash->{'body'},0,length($hash->{'body'})-length(@ImgNamTeile[@ImgNamTeile-1]));
    
### Image-Pfad auf Existenz prfen
    foreach $tmp (@Image)
    {
     my $temp = $img_pfad.$tmp;
     unless (-e $temp) {return("Fehler in Bilddatei fr Body: $tmp, Fehler: $!")};
    }    
   }
  }
  
### HTML-Body-Text durch "cid:" in Bildlinks ergnzen
  $body =~ s/<img src=\"(.*?)\"/<img src=\"cid:$1\"/g;
 }

##############################################
### Mailobjekt anlegen

$msg = new MIME::Lite;
$msg->build(
           From         =>$hash->{'from'},                # Wichtig, alle Variablen ohne Quotierung!!
           To           =>(join(',', @{$hash->{'to'}})),
           "Reply-To"   =>$hash->{'reply'},
           Sender       =>$hash->{'from'},
           Subject      =>"$hash->{subject}",
           #Type        =>'multipart/related'
           Type        =>'multipart/mixed'
           #Type         =>'multipart/alternative', # Fuer Outlook-Express OK!
           );

### Cc ergnzen, falls vorhanden
if ($hash->{'cc'})
{
 $msg->add(Cc           =>(join(',', @{$hash->{'cc'}})));
}

### Bcc ergnzen, falls vorhanden
if ($hash->{'bcc'})
{
 $msg->add(Bcc           =>(join(',', @{$hash->{'bcc'}})));
}

### Prioritt festlegen, falls vorhanden
if ($hash->{'imp'})
{
 if ($hash->{'imp'} eq "high")                    # rfc2076, rfc0821
 {
  $msg->add(Priority            =>    'urgent');
  $msg->add(Importance          =>    'High');
  $msg->add(Precedence          =>    'first-class');
  $msg->add(Importance          =>    'High');
  $msg->add('X-Priority'        =>    '1');    # wird nicht untersttzt, setzt rote Fahne bei Outlook-Express
  $msg->add('X-MSMail-Priority' =>'High');     # wird nicht untersttzt, setzt rote Fahne bei Outlook-Express
 }
 elsif ($hash->{'imp'} eq "low")
 {
  $msg->add(Priority            =>    'non-urgent');
  $msg->add(Importance          =>    'low');
  $msg->add(Importance          =>    'Low');
  $msg->add('X-Priority'        =>    '5');    # wird nicht untersttzt, setzt blauen Pfeil bei Outlook-Express
  $msg->add('X-MSMail-Priority' =>'Low');     # wird nicht untersttzt, setzt blauen Pfeil bei Outlook-Express
 }
 
}
           
 $msg->attach(Type => "$bodytyp",
              Data => qq{ $body	 }
             );
    
### Attachements fr eingelagerte Bilder erzeugen
### - fr mehrfach vorhandene Bilder jeweils nur ein Attachement
if ($bodytyp eq 'text/html' && @Image > 0)
{
 my @Img = sort @Image;
 $tmp = "";
 foreach my $Img (@Img)
 {
  unless ($tmp eq $Img)
  {
   $msg->attach(Type => 'AUTO',
                Id   => "$Img",
                Path => "$img_pfad$Img",
                );
   $tmp = $Img;
  }
 }
}

#### Dateianhnge hinzufgen
if ($hash->{'attach'} && @{$hash->{'attach'}} > 0)
{
 foreach my $attach (@{$hash->{'attach'}})
 {
 # Prfen ob Datei fr Anhang existiert
  unless (-e $attach) {return("Fehler in Datei fr Anhang: $attach, Fehler: $!")};
  if ($system eq "smtp")
  {
   @DatNamTeile = split(/\\/,$attach); # Dateipfad bei "\" aufsplitten
  }
  else
  {
   @DatNamTeile = split(/\//,$attach); # Dateipfad bei "/" aufsplitten
  }  
  my $file_name    = @DatNamTeile[@DatNamTeile-1];  # letztes Element ist Dateiname


$msg->attach(Type     =>'AUTO',
             Path     =>"$attach",
             Filename =>"$file_name",
          Disposition => 'attachment'
            );
 }
}

### Mail abschicken
if ($system eq "smtp")
{
 MIME::Lite->send('smtp', $smtp_server, Timeout=>60);
}
else
{
 MIME::Lite->send('sendmail', '/usr/sbin/sendmail -t');
}

if ($msg->send)
{
 return("OK - Mailaussendung abgeschlossen");
}
else
{
 return("Fehler in der Mailaussendung: $!");
}
}
#######################################################
# EMAIL_TEST
# input:	Emailadresse
# output:	'true' oder 'false'
#
# Prft Emailadresse auf Plausibilitt
#######################################################	
sub EMAIL_TEST
{
 my $adr = $_[0];
 if ($adr =~ /[\w\.\-=]+@[\w\.\-]+\.\w{2,}/g)
 {
  return ('true');
 }
 else
 {
  return ('false');
 }
}
##################################################################
# GET_DATEI_STRING
# Liest Datei als String ein (Schlrfmodus)
#
# input:  absoluter Pfad zur Datei
#
# output: vollstndiger Pfad
##################################################################
sub GET_DATEI_STRING
{
 my $Datei=$_[0];
 my $DateiInhalt = '';
 if (-e $Datei)
 {
  open (THEFILE, "< $Datei") || die "Kann Datei $Datei nicht oeffen: $!\n";
  local $/; # Schlrfmodus einschlaten
  while (<THEFILE>) 
  {
   $DateiInhalt = $_;  # gesamte Datei drin!
  }
  close (THEFILE) || die "Can't close the text file: $!\n";
 }    
else {die "Datei $Datei kann nicht geoeffnet werden";} 
return ($DateiInhalt);
}
